import * as dotenv from "dotenv";
import * as path from "path";

import { Sequelize, Dialect } from "sequelize";
import { Subject } from "rxjs";

import Log from "../../libs/log";

const models: any = [];

export default class SequelizeDbConnector {
  private static _instance: SequelizeDbConnector;
  private _isTestingEnv = false;
  private _log: Log;

  public sequelize: Sequelize;
  public ready: Subject<boolean>;

  private constructor() {
    this._isTestingEnv = process.env.NODE_ENV == "test";
    this._log = Log.instance;

    this.ready = new Subject();
    this.ready.next(false);

    const basePath = path.join(__dirname, "../../");
    const confPath = this._isTestingEnv
      ? `${basePath}test/conf.test.env`
      : `${basePath}conf.env`;

    const result = dotenv.config({
      path: confPath
    });

    if (result.error) {
      throw new Error("Error retrieving config");
    }

    const client = process.env.DB_CLIENT || "mysql";
    const host = process.env.DB_HOST;
    const port = Number(process.env.DB_PORT);
    const user = process.env.DB_USER;
    const password = process.env.DB_PASS;
    const schema = process.env.DB_SCHEMA;

    this.sequelize = new Sequelize(schema, user, password, {
      host,
      port,
      dialect: client as Dialect,
      logging: (sql: string, timming?: number) => {
        if (!this._isTestingEnv) {
          this._log.logger.debug(sql);
          this._log.logger.debug(timming);
        }
      }
    });

    this.sequelize
      .authenticate()
      .then(() => {
        if (!this._isTestingEnv) {
          this._log.logger.debug(`"${client}" connection success!`);
        }
      })
      .catch(err => {
        this._log.logger.fatal(err.message);
        process.exit(1);
      });

    this.initModels();
  }

  private async initModels() {
    models.forEach((model: any) => {
      // this._log.logger.info(`Init model "${model.name}"`);
      model.inits(this.sequelize);
    });

    models.forEach((model: any) => {
      if (model.associate) {
        model.associate(this.sequelize);
      }
    });

    // await this.sequelize.sync({ alter: true });
    this.ready.next(true);
    this.ready.complete();
  }

  public static get instance() {
    return this._instance || (this._instance = new this());
  }
}
