import * as dotenv from "dotenv";
import * as http from "http";
import * as fs from "fs";
import * as path from "path";
import { Express } from "express";

import ExpressApp from "./frameworks/express/express-app";

import Log from "./libs/log";

class HttpServer {
  private static _instance: HttpServer;
  private _server: http.Server;
  private _expressApp: Express;
  private _basePath: string;
  private _isTestingEnv = false;
  private _log: Log;
  
  private constructor() {
    try {
      this._isTestingEnv = process.env.NODE_ENV == "test";

      this.initConfig();
      this.initLog();
      this.initExpressApp();
      this.initHttpServer();
      this.initFolders();
    } catch (err) {
      console.error(err.message);
      process.exit(1);
    }
  }

  private initConfig() {
    this._basePath = path.join(__dirname, "./");
    const confPath = this._isTestingEnv
      ? `${this._basePath}test/conf.test.env`
      : `${this._basePath}conf.env`;

    const result = dotenv.config({
      path: confPath
    });

    if (result.error) {
      console.error(result.error);
      process.exit(1);
    }

    if (process.env.ENVIRONMENT !== "prod") {
      process.env["NODE_TLS_REJECT_UNAUTHORIZED"] = "0";
    }
  }

  private initLog() {
    this._log = Log.instance;
    this._log.logger.level = process.env.LOG_LEVEL;
  }

  private initExpressApp() {
    this._expressApp = ExpressApp.instance;
  }

  private initHttpServer() {
    if (!this._expressApp) {
      throw new Error("App not initialized");
    }

    const port = Number(process.env.PORT) || 3000;
    this._server = http.createServer(this._expressApp);
    this._server.listen(port, () => {
      if (typeof process.send != "undefined") {
        process.send("ready");
      }
      return this._log.logger.info(`Server listening on port ${port}`);
    });
  }

  private initFolders() {
    if (!fs.existsSync(`${this._basePath}public`)) {
      fs.mkdirSync(`${this._basePath}public`);
    }
  }

  public static get instance() {
    return this._instance || (this._instance = new this());
  }

  public get expressApp() {
    return this._expressApp;
  }
}

const server = HttpServer.instance;
export { server };
