import * as dotenv from "dotenv";
import * as path from "path";
import * as fs from "fs";
import * as log4js from "log4js";

export default class Log {
  private static _instance: Log;
  private _isTestingEnv: boolean;
  private _logger: log4js.Logger;

  private constructor() {
    const logsDirectory = path.join(__dirname, "../../logs");

    if (!fs.existsSync(logsDirectory)) {
      fs.mkdirSync(logsDirectory);
    }

    this.initConfig();
    this.configureLog(logsDirectory);
  }

  private initConfig() {
    this._isTestingEnv = process.env.NODE_ENV == "test";

    const basePath = path.join(__dirname, "../../");
    const confPath = this._isTestingEnv
      ? `${basePath}test/conf.test.env`
      : `${basePath}conf.env`;

    const result = dotenv.config({
      path: confPath
    });

    if (result.error) {
      throw new Error("Error retrieving config");
    }
  }

  private configureLog(logsDirectory: string) {
    const today = new Date();
    const month = (today.getMonth() + 1).toString().padStart(2, "0");
    const logFileName = `/logs-${today.getFullYear()}${month}${today.getDate()}.log`;

    log4js.configure({
      appenders: {
        app: {
          backups: 5,
          compress: true,
          filename: logsDirectory.concat(logFileName),
          layout: {
            pattern: "[%d{dd/MM/yyyy hh:mm:ss,SSS}] %p [%z] %m%n",
            type: "pattern"
          },
          maxLogSize: 204857600, // 200MB
          type: "file"
        },
        out: { type: "stdout", layout: { type: "coloured" } }
      },
      categories: {
        default: { appenders: ["out", "app"], level: "debug" }
      }
    });

    this._logger = log4js.getLogger();

    // console.debug("LogLevel", process.env.LOG_LEVEL);
    this._logger.level = process.env.LOG_LEVEL;
  }

  public static get instance() {
    return this._instance || (this._instance = new this());
  }

  public get logger() {
    return this._logger;
  }
}
